@@ -18,20 +18,19 @@
 package gengo
 
 import (
-	"bytes"
-	"errors"
 	"fmt"
 	"os"
-	"os/exec"
-	"path"
+	"strings"
 
 	"github.com/goplus/gop/cl"
 	"github.com/goplus/gop/cmd/gengo"
 	"github.com/goplus/gop/cmd/internal/base"
+	"github.com/goplus/gox"
+	"github.com/qiniu/x/log"
 )
 
 // -----------------------------------------------------------------------------
-
+/*
 var (
 	errTestFailed = errors.New("test failed")
 )
@@ -64,19 +63,20 @@ func testPkg(p *gengo.Runner, dir string, flags int) error {
 	}
 	return nil
 }
-
+*/
 // -----------------------------------------------------------------------------
 
 // Cmd - gop go
 var Cmd = &base.Command{
-	UsageLine: "gop go [-test -slow] <gopSrcDir>",
+	UsageLine: "gop go [-debug -test -slow] <gopSrcDir>",
 	Short:     "Convert Go+ packages into Go packages",
 }
 
 var (
-	flag     = &Cmd.Flag
-	flagTest = flag.Bool("test", false, "test Go+ package")
-	flagSlow = flag.Bool("slow", false, "don't cache imported packages")
+	flag      = &Cmd.Flag
+	flagDebug = flag.Bool("debug", false, "set log level to debug")
+	flagTest  = flag.Bool("test", false, "test Go+ package")
+	flagSlow  = flag.Bool("slow", false, "don't cache imported packages")
 )
 
 func init() {
@@ -89,7 +89,12 @@ func runCmd(cmd *base.Command, args []string) {
 		cmd.Usage(os.Stderr)
 		return
 	}
+	if *flagDebug {
+		log.SetOutputLevel(log.Ldebug)
+		gox.SetDebug(gox.DbgFlagAll)
+	}
 	dir := flag.Arg(0)
+	dir = strings.TrimSuffix(dir, "/...")
 	runner := new(gengo.Runner)
 	runner.SetAfter(func(p *gengo.Runner, dir string, flags int) error {
 		errs := p.ResetErrors()
