@@ -18,12 +18,12 @@
 package run
 
 import (
+	"crypto/sha1"
+	"encoding/base64"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"os/exec"
 	"path/filepath"
-	"strings"
 
 	"github.com/goplus/gop/ast"
 	"github.com/goplus/gop/cl"
@@ -41,7 +41,7 @@ import (
 
 // Cmd - gop run
 var Cmd = &base.Command{
-	UsageLine: "gop run [-asm -quiet -debug -prof] <gopSrcDir|gopSrcFile>",
+	UsageLine: "gop run [-asm -quiet -debug -nr -gop -prof] <gopSrcDir|gopSrcFile>",
 	Short:     "Run a Go+ program",
 }
 
@@ -51,6 +51,8 @@ var (
 	flagVerbose = flag.Bool("v", false, "print verbose information")
 	flagQuiet   = flag.Bool("quiet", false, "don't generate any compiling stage log")
 	flagDebug   = flag.Bool("debug", false, "set log level to debug")
+	flagNorun   = flag.Bool("nr", false, "don't run if no change")
+	flagGop     = flag.Bool("gop", false, "parse a .go file as a .gop file")
 	flagProf    = flag.Bool("prof", false, "do profile and generate profile report")
 )
 
@@ -118,81 +120,102 @@ func runCmd(cmd *base.Command, args []string) {
 
 	fset := token.NewFileSet()
 	src, _ := filepath.Abs(flag.Arg(0))
-	isDir, err := IsDir(src)
+	fi, err := os.Stat(src)
 	if err != nil {
 		log.Fatalln("input arg check failed:", err)
 	}
-	if !isDir && filepath.Ext(src) == ".go" { // not a Go+ file
-		runGoFile(src, args)
-		return
-	}
+	isDir := fi.IsDir()
 
+	var isDirty bool
 	var srcDir, file, gofile string
 	var pkgs map[string]*ast.Package
 	if isDir {
 		srcDir = src
 		gofile = src + "/gop_autogen.go"
-		pkgs, err = parser.ParseDir(fset, src, nil, 0)
+		isDirty = true // TODO: check if code changed
+		if isDirty {
+			pkgs, err = parser.ParseDir(fset, src, nil, 0)
+		} else if *flagNorun {
+			return
+		}
 	} else {
 		srcDir, file = filepath.Split(src)
-		if hasMultiFiles(srcDir, ".gop") {
+		isGo := filepath.Ext(file) == ".go"
+		if isGo {
+			hash := sha1.Sum([]byte(src))
+			dir := os.Getenv("HOME") + "/.gop/run"
+			os.MkdirAll(dir, 0777)
+			gofile = dir + "/g" + base64.RawURLEncoding.EncodeToString(hash[:]) + file
+		} else if hasMultiFiles(srcDir, ".gop") {
 			gofile = filepath.Join(srcDir, "gop_autogen_"+file+".go")
 		} else {
 			gofile = srcDir + "/gop_autogen.go"
 		}
-		pkgs, err = parser.Parse(fset, src, nil, 0)
+		isDirty = fileIsDirty(fi, gofile)
+		if isDirty {
+			if isGo {
+				fmt.Println("==> GenGo to", gofile)
+			}
+			if *flagGop {
+				pkgs, err = parser.Parse(fset, src, nil, 0)
+			} else {
+				pkgs, err = parser.Parse(fset, src, nil, 0) // TODO: only to check dependencies
+			}
+		} else if *flagNorun {
+			return
+		}
 	}
 	if err != nil {
 		scanner.PrintError(os.Stderr, err)
 		os.Exit(10)
 	}
 
-	mainPkg, ok := pkgs["main"]
-	if !ok {
-		if len(pkgs) == 0 { // not a Go+ package, try runGoPkg
-			runGoPkg(src, args, true)
-			return
+	if isDirty {
+		mainPkg, ok := pkgs["main"]
+		if !ok {
+			if len(pkgs) == 0 && isDir { // not a Go+ package, try runGoPkg
+				runGoPkg(src, args, true)
+				return
+			}
+			fmt.Fprintln(os.Stderr, "TODO: not a main package")
+			os.Exit(12)
 		}
-		fmt.Fprintln(os.Stderr, "TODO: not a main package")
-		os.Exit(12)
+
+		modDir, noCacheFile := findGoModDir(srcDir)
+		conf := &cl.Config{
+			Dir: modDir, TargetDir: srcDir, Fset: fset, CacheLoadPkgs: true, PersistLoadPkgs: !noCacheFile}
+		out, err := cl.NewPackage("", mainPkg, conf)
+		if err != nil {
+			fmt.Fprintln(os.Stderr, err)
+			os.Exit(11)
+		}
+		err = saveGoFile(gofile, out)
+		if err != nil {
+			log.Fatalln("saveGoFile failed:", err)
+		}
+		conf.PkgsLoader.Save()
 	}
 
-	modDir, noCacheFile := findGoModDir(srcDir)
-	conf := &cl.Config{
-		Dir: modDir, TargetDir: srcDir, Fset: fset, CacheLoadPkgs: true, PersistLoadPkgs: !noCacheFile}
-	out, err := cl.NewPackage("", mainPkg, conf)
-	if err != nil {
-		fmt.Fprintln(os.Stderr, err)
-		os.Exit(11)
-	}
-	err = saveGoFile(gofile, out)
-	if err != nil {
-		log.Fatalln("saveGoFile failed:", err)
-	}
-	conf.PkgsLoader.Save()
 	goRun(gofile, args)
 	if *flagProf {
 		panic("TODO: profile not impl")
 	}
 }
 
-// IsDir checks a target path is dir or not.
-func IsDir(target string) (bool, error) {
-	fi, err := os.Stat(target)
+func fileIsDirty(fi os.FileInfo, gofile string) bool {
+	fiDest, err := os.Stat(gofile)
 	if err != nil {
-		return false, err
+		return true
 	}
-	return fi.IsDir(), nil
+	return fi.ModTime().After(fiDest.ModTime())
 }
 
-func goRun(target string, args []string) {
-	dir, file := filepath.Split(target)
+func goRun(file string, args []string) {
 	goArgs := make([]string, len(args)+2)
 	goArgs[0] = "run"
 	goArgs[1] = file
 	copy(goArgs[2:], args)
 	cmd := exec.Command("go", goArgs...)
-	cmd.Dir = dir
 	cmd.Stdin = os.Stdin
 	cmd.Stdout = os.Stdout
 	cmd.Stderr = os.Stderr
@@ -201,7 +224,6 @@ func goRun(target string, args []string) {
 	if err != nil {
 		switch e := err.(type) {
 		case *exec.ExitError:
-			os.Stderr.Write(e.Stderr)
 			os.Exit(e.ExitCode())
 		default:
 			log.Fatalln("go run failed:", err)
@@ -247,27 +269,6 @@ func runGoPkg(src string, args []string, doRun bool) {
 	}
 }
 
-func runGoFile(src string, args []string) {
-	targetDir, file := filepath.Split(src)
-	if hasMultiFiles(targetDir, ".go") {
-		targetDir = filepath.Join(targetDir, ".gop", strings.TrimSuffix(file, ".go"))
-		gofile := filepath.Join(targetDir, "gop_autogen.go")
-		b, err := ioutil.ReadFile(src)
-		if err != nil {
-			log.Fatalln("ReadFile failed:", err)
-		}
-		os.MkdirAll(targetDir, 0777)
-		err = ioutil.WriteFile(gofile, b, 0666)
-		if err != nil {
-			log.Fatalln("WriteFile failed:", err)
-		}
-		runGoPkg(targetDir, args, false)
-		goRun(src, args)
-	} else {
-		runGoPkg(targetDir, args, true)
-	}
-}
-
 func hasMultiFiles(srcDir string, ext string) bool {
 	var has bool
 	if f, err := os.Open(srcDir); err == nil {
